perl -I../lib dn_check_structure.t 1
perl -I../lib dn_check_value.t 1
perl -I../lib dn_delim.t 1
perl -I../lib dn_empty_1.t 1
perl -I../lib dn_empty_2.t 1
perl -I../lib dn_erase_1.t 1
perl -I../lib dn_erase_2.t 1
perl -I../lib dn_erase_3.t 1
perl -I../lib dn_get_merge.t 1
perl -I../lib dn_get_structure.t 1
perl -I../lib dn_identical.t 1
perl -I../lib dn_keys.t 1
perl -I../lib dn_merge_1.t 1
perl -I../lib dn_merge_2.t 1
perl -I../lib dn_merge_path_1.t 1
perl -I../lib dn_merge_path_2.t 1
perl -I../lib dn_merge_path_3.t 1
perl -I../lib dn_nds_1.t 1
perl -I../lib dn_nds_2.t 1
perl -I../lib dn_nds_3.t 1
perl -I../lib dn_path_1.t 1
perl -I../lib dn_path_2.t 1
perl -I../lib dn_paths.t 1
perl -I../lib dn_ruleset.t 1
perl -I../lib dn_ruleset_valid.t 1
perl -I../lib dn_set_merge.t 1
perl -I../lib dn_set_structure.t 1
perl -I../lib dn_test_conditions.hash.t 1
perl -I../lib dn_test_conditions.list.t 1
perl -I../lib dn_test_conditions.scalar.t 1
perl -I../lib dn_value.t 1
perl -I../lib dn_values.t 1
perl -I../lib dn_which_1.t 1
perl -I../lib dn_which_2.t 1
perl -I../lib me.add_ele.hash.t 1
perl -I../lib me.add_ele.list.t 1
perl -I../lib me.add_ele.olist.t 1
perl -I../lib me.copy_hash.t 1
perl -I../lib me.copy_list.t 1
perl -I../lib me.default_element.hash.t 1
perl -I../lib me.default_element.list.t 1
perl -I../lib me.default_element.olist.t 1
perl -I../lib me.delete_ele.hash.t 1
perl -I../lib me.delete_ele.list.t 1
perl -I../lib me.ele.hash.t 1
perl -I../lib me.ele.list.t 1
perl -I../lib me.eles.hash.t 1
perl -I../lib me.eles.list.t 1
perl -I../lib me.file.hash.error.t 1
perl -I../lib me.file.hash.t 1
perl -I../lib me.file.list.error.t 1
perl -I../lib me.file.list.t 1
perl -I../lib me.is_default_value.hash.t 1
perl -I../lib me.keys.t 1
perl -I../lib me.path_in_use.t 1
perl -I../lib me.path_valid.t 1
perl -I../lib me.path_values.hash.t 1
perl -I../lib me.path_values.list.t 1
perl -I../lib me.rename_ele.hash.t 1
perl -I../lib me.rename_ele.olist.t 1
perl -I../lib me.update_ele.t 1
perl -I../lib me.value.t 1
perl -I../lib me.values.t 1
perl -I../lib me.which.hash.t 1
perl -I../lib me.which.list.t 1
perl -I../lib mf.add_ele.hash.t 1
perl -I../lib mf.add_ele.list.t 1
perl -I../lib mf.copy_hash.t 1
perl -I../lib mf.copy_list.t 1
perl -I../lib mf.default_element.hash.t 1
perl -I../lib mf.default_element.list.t 1
perl -I../lib mf.delete_ele.hash.t 1
perl -I../lib mf.delete_ele.list.t 1
perl -I../lib mf.ele.hash.t 1
perl -I../lib mf.ele.list.t 1
perl -I../lib mf.eles.hash.t 1
perl -I../lib mf.eles.list.t 1
perl -I../lib mf.file.hash.error.t 1
perl -I../lib mf.file.hash.t 1
perl -I../lib mf.file.list.error.t 1
perl -I../lib mf.file.list.t 1
perl -I../lib mf.is_default_value.hash.t 1
perl -I../lib mf.keys.t 1
perl -I../lib mf.path_in_use.t 1
perl -I../lib mf.path_valid.t 1
perl -I../lib mf.path_values.hash.t 1
perl -I../lib mf.path_values.list.t 1
perl -I../lib mf.rename_ele.hash.t 1
perl -I../lib mf.update_ele.t 1
perl -I../lib mf.value.t 1
perl -I../lib mf.values.t 1
perl -I../lib mf.which.hash.t 1
perl -I../lib mf.which.list.t 1
